/**
 * Creates a factory function for props definitions.
 * This is used to define props in a composable then override
 * default values in an implementing component.
 *
 * @example Simplified signature
 * (props: Props) => (defaults?: Record<keyof props, any>) => Props
 *
 * @example Usage
 * const makeProps = propsFactory({
 *   foo: String,
 * })
 *
 * defineComponent({
 *   props: {
 *     ...makeProps({
 *       foo: 'a',
 *     }),
 *   },
 *   setup (props) {
 *     // would be "string | undefined", now "string" because a default has been provided
 *     props.foo
 *   },
 * }
 */
export default function propsFactory(props) {
  return defaults => {
    if (!defaults) {
      return props;
    } else {
      return Object.keys(props).reduce((obj, prop) => {
        const definition = props[prop];

        if (prop in defaults) {
          if (typeof definition === 'object' && definition != null && !Array.isArray(definition)) {
            obj[prop] = { ...definition,
              default: defaults[prop]
            };
          } else {
            obj[prop] = {
              type: props[prop],
              default: defaults[prop]
            };
          }
        } else {
          obj[prop] = definition;
        }

        return obj;
      }, {});
    }
  };
}
//# sourceMappingURL=propsFactory.mjs.map